@@ -72,18 +72,22 @@ class Check(Reporting):
     def _flags(self) -> List[str]:
         flags = super()._flags()
         filter_directories = self._get_directories_to_analyze()
+        filter_directories.update(
+            set(self._configuration.get_existent_do_not_ignore_errors_in_paths())
+        )
         if len(filter_directories):
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
         flags.extend(["-workers", str(self._configuration.number_of_workers)])
         search_path = self._configuration.search_path + typeshed_search_path(
             self._configuration.typeshed
         )
-        if len(self._configuration.ignore_all_errors):
+
+        ignore_all_errors_paths = (
+            self._configuration.get_existent_ignore_all_errors_paths()
+        )
+        if len(ignore_all_errors_paths):
             flags.extend(
-                [
-                    "-ignore-all-errors",
-                    ";".join(sorted(self._configuration.ignore_all_errors)),
-                ]
+                ["-ignore-all-errors", ";".join(sorted(ignore_all_errors_paths))]
             )
         if search_path:
             flags.extend(["-search-path", ",".join(search_path)])
