@@ -50,6 +50,7 @@ class ExitCode(enum.IntEnum):
     BUCK_ERROR = 3
     SERVER_NOT_FOUND = 4
     INCONSISTENT_SERVER = 5
+    CONFIGURATION_ERROR = 6
     # If the process exited due to a signal, this will be the negative signal number.
     SIGSEGV = -signal.SIGSEGV
 
@@ -377,6 +378,11 @@ class Command(CommandParser, ABC):
             analysis_directory or self.generate_analysis_directory()
         )
 
+        if self._command_arguments.filter_directory is not None:
+            LOG.warning(
+                "Warning: `--filter-directory` is deprecated and will be removed soon."
+            )
+
     @classmethod
     def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
         pass
