@@ -164,14 +164,18 @@ class Start(Reporting):
             for path in self._taint_models_path:
                 flags.extend(["-taint-models", path])
         filter_directories = self._get_directories_to_analyze()
+        filter_directories.update(
+            set(self._configuration.get_existent_do_not_ignore_errors_in_paths())
+        )
         if len(filter_directories):
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
-        if len(self._configuration.ignore_all_errors):
+
+        ignore_all_errors_paths = (
+            self._configuration.get_existent_ignore_all_errors_paths()
+        )
+        if len(ignore_all_errors_paths):
             flags.extend(
-                [
-                    "-ignore-all-errors",
-                    ";".join(sorted(self._configuration.ignore_all_errors)),
-                ]
+                ["-ignore-all-errors", ";".join(sorted(ignore_all_errors_paths))]
             )
         if self._terminal:
             flags.append("-terminal")
