@@ -36,6 +36,13 @@ from .resources import LOG_DIRECTORY
 LOG: Logger = logging.getLogger(__name__)
 
 
+def _relativize_root(root: str, project_root: str, relative_root: Optional[str]) -> str:
+    if root.startswith("//"):
+        return expand_relative_path(project_root, root[2:])
+    else:
+        return expand_relative_path(relative_root or "", root)
+
+
 class InvalidConfiguration(Exception):
     def __init__(self, message: str) -> None:
         self.message = f"Invalid configuration: {message}"
@@ -55,9 +62,7 @@ class SearchPathElement:
     ) -> "SearchPathElement":
         if isinstance(path, str):
             return SearchPathElement(
-                root=SearchPathElement.relativize_root(
-                    path, project_root, path_relative_to
-                ),
+                _relativize_root(path, project_root, path_relative_to),
                 subdirectory=None,
             )
         else:
@@ -65,9 +70,7 @@ class SearchPathElement:
                 root = path["root"]
                 subdirectory = path["subdirectory"]
                 return SearchPathElement(
-                    root=SearchPathElement.relativize_root(
-                        root, project_root, path_relative_to
-                    ),
+                    _relativize_root(root, project_root, path_relative_to),
                     subdirectory=subdirectory,
                 )
             elif "site-package" in path:
@@ -77,9 +80,7 @@ class SearchPathElement:
                 found_element = None
                 for root in site_root:
                     site_package_element = SearchPathElement(
-                        root=SearchPathElement.relativize_root(
-                            root, project_root, None
-                        ),
+                        _relativize_root(root, project_root, None),
                         subdirectory=subdirectory,
                     )
                     if os.path.isdir(site_package_element.path()):
@@ -95,15 +96,6 @@ class SearchPathElement:
                     "specified."
                 )
 
-    @staticmethod
-    def relativize_root(
-        root: str, project_root: str, relative_root: Optional[str]
-    ) -> str:
-        if root.startswith("//"):
-            return expand_relative_path(project_root, root[2:])
-        else:
-            return expand_relative_path(relative_root or "", root)
-
     def path(self) -> str:
         subdirectory = self.subdirectory
         if subdirectory is not None:
@@ -161,7 +153,7 @@ class _ConfigurationFile:
 
         value = self._configuration.pop(key, default)
         if raise_on_override and current and value:
-            raise EnvironmentException(
+            raise InvalidConfiguration(
                 f"Configuration file may not override `{key}` field."
             )
         if current:
@@ -214,7 +206,7 @@ class Configuration:
         self.targets = []
         self.logger = logger
         self.formatter = formatter
-        self.ignore_all_errors = []
+        self._ignore_all_errors = []
         self.number_of_workers: int = 0
         self.project_root: str = project_root
         self._local_root: Optional[str] = local_root
@@ -241,6 +233,7 @@ class Configuration:
         self._use_buck_builder: Optional[bool] = use_buck_builder
         self._use_buck_source_database: Optional[bool] = use_buck_source_database
         self.ignore_infer: List[str] = []
+        self._do_not_ignore_errors_in: List[str] = []
 
         self._search_path: List[SearchPathElement] = []
         if search_path:
@@ -267,7 +260,6 @@ class Configuration:
 
         if local_root:
             self._check_read_local_configuration(local_root)
-        self.ignore_all_errors.append(os.path.abspath(self.log_directory))
 
         self.autocomplete = False
 
@@ -316,134 +308,102 @@ class Configuration:
         )
 
     def _validate(self) -> None:
-        try:
-
-            def is_list_of_strings(list):
-                if len(list) == 0:
-                    return True
-                return not isinstance(list, str) and all(
-                    isinstance(element, str) for element in list
-                )
-
-            if not is_list_of_strings(
-                self.source_directories
-            ) or not is_list_of_strings(self.targets):
-                raise InvalidConfiguration(
-                    "`target` and `source_directories` fields must be lists of "
-                    "strings."
-                )
-
-            if not is_list_of_strings(self.ignore_all_errors):
-                raise InvalidConfiguration(
-                    "`ignore_all_errors` field must be a list of strings."
-                )
-
-            if not is_list_of_strings(self.ignore_infer):
-                raise InvalidConfiguration(
-                    "`ignore_infer` field must be a list of strings."
-                )
-
-            if not is_list_of_strings(self.extensions):
-                raise InvalidConfiguration(
-                    "`extensions` field must be a list of strings."
-                )
-            if not all(
-                extension.startswith(".") or not extension
-                for extension in self.extensions
-            ):
-                raise InvalidConfiguration(
-                    "`extensions` must only contain strings formatted as `.EXT`"
-                )
-
-            if not os.path.exists(self.binary):
-                raise InvalidConfiguration(f"Binary at `{self.binary}` does not exist.")
-
-            if self.number_of_workers < 1:
-                raise InvalidConfiguration("Number of workers must be greater than 0.")
-
-            # Validate typeshed path and sub-elements.
-            assert_readable_directory_in_configuration(
-                self.typeshed, field_name="typeshed"
+        def is_list_of_strings(list):
+            if len(list) == 0:
+                return True
+            return not isinstance(list, str) and all(
+                isinstance(element, str) for element in list
             )
 
-            # A courtesy warning since we have changed default behaviour.
-            if self._typeshed_has_obsolete_value():
-                LOG.warning(
-                    f"It appears that `{self.typeshed}` points at a `stdlib` "
-                    "directory. Please note that the `typeshed` configuration must "
-                    "point to the root of the `typeshed` directory."
-                )
+        if not is_list_of_strings(self.source_directories) or not is_list_of_strings(
+            self.targets
+        ):
+            raise InvalidConfiguration(
+                "`target` and `source_directories` fields must be lists of " "strings."
+            )
 
-            expanded_ignore_paths = []
-            for path in self.ignore_all_errors:
-                expanded = glob.glob(path)
-                if not expanded:
-                    expanded_ignore_paths.append(path)
-                else:
-                    expanded_ignore_paths += expanded
-            self.ignore_all_errors = expanded_ignore_paths
+        if not is_list_of_strings(self._ignore_all_errors):
+            raise InvalidConfiguration(
+                "`ignore_all_errors` field must be a list of strings."
+            )
 
-            non_existent_ignore_paths = [
-                path for path in self.ignore_all_errors if not os.path.exists(path)
+        if not is_list_of_strings(self.ignore_infer):
+            raise InvalidConfiguration(
+                "`ignore_infer` field must be a list of strings."
+            )
+
+        if not is_list_of_strings(self.extensions):
+            raise InvalidConfiguration("`extensions` field must be a list of strings.")
+        if not all(
+            extension.startswith(".") or not extension for extension in self.extensions
+        ):
+            raise InvalidConfiguration(
+                "`extensions` must only contain strings formatted as `.EXT`"
+            )
+
+        if not os.path.exists(self.binary):
+            raise InvalidConfiguration(f"Binary at `{self.binary}` does not exist.")
+
+        if self.number_of_workers < 1:
+            raise InvalidConfiguration("Number of workers must be greater than 0.")
+
+        # Validate typeshed path and sub-elements.
+        assert_readable_directory_in_configuration(self.typeshed, field_name="typeshed")
+
+        # A courtesy warning since we have changed default behaviour.
+        if self._typeshed_has_obsolete_value():
+            LOG.warning(
+                f"It appears that `{self.typeshed}` points at a `stdlib` "
+                "directory. Please note that the `typeshed` configuration must "
+                "point to the root of the `typeshed` directory."
+            )
+
+        non_existent_infer_paths = [
+            path for path in self.ignore_infer if not os.path.exists(path)
+        ]
+        if non_existent_infer_paths:
+            LOG.warning(
+                "Nonexistent paths passed in to `ignore_infer`: "
+                f"`{non_existent_infer_paths}`"
+            )
+            self.ignore_infer = [
+                path
+                for path in self.ignore_infer
+                if path not in non_existent_infer_paths
             ]
-            if non_existent_ignore_paths:
-                LOG.warning(
-                    "Nonexistent paths passed in to `ignore_all_errors`: "
-                    f"`{non_existent_ignore_paths}`"
-                )
-                self.ignore_all_errors = [
-                    path
-                    for path in self.ignore_all_errors
-                    if path not in non_existent_ignore_paths
-                ]
 
-            non_existent_infer_paths = [
-                path for path in self.ignore_infer if not os.path.exists(path)
-            ]
-            if non_existent_infer_paths:
-                LOG.warning(
-                    "Nonexistent paths passed in to `ignore_infer`: "
-                    f"`{non_existent_infer_paths}`"
-                )
-                self.ignore_infer = [
-                    path
-                    for path in self.ignore_infer
-                    if path not in non_existent_infer_paths
-                ]
-
-            for typeshed_subdirectory_name in ["stdlib", "third_party"]:
-                typeshed_subdirectory = os.path.join(
-                    self.typeshed, typeshed_subdirectory_name
-                )
-                assert_readable_directory_in_configuration(typeshed_subdirectory)
-                for typeshed_version_directory_name in os.listdir(
-                    typeshed_subdirectory
-                ):
-                    if not typeshed_version_directory_name[0].isdigit():
-                        raise InvalidConfiguration(
-                            "Directories inside `typeshed` must only contain "
-                            "second-level subdirectories starting with "
-                            "a version number."
-                        )
-                    typeshed_version_directory = os.path.join(
-                        typeshed_subdirectory, typeshed_version_directory_name
+        for typeshed_subdirectory_name in ["stdlib", "third_party"]:
+            typeshed_subdirectory = os.path.join(
+                self.typeshed, typeshed_subdirectory_name
+            )
+            assert_readable_directory_in_configuration(typeshed_subdirectory)
+            for typeshed_version_directory_name in os.listdir(typeshed_subdirectory):
+                if not typeshed_version_directory_name[0].isdigit():
+                    raise InvalidConfiguration(
+                        "Directories inside `typeshed` must only contain "
+                        "second-level subdirectories starting with "
+                        "a version number."
                     )
-                    assert_readable_directory_in_configuration(
-                        typeshed_version_directory
-                    )
-
-            # Validate elements of the search path.
-            for element in self._search_path:
-                assert_readable_directory_in_configuration(
-                    element.path(), field_name="search_path"
+                typeshed_version_directory = os.path.join(
+                    typeshed_subdirectory, typeshed_version_directory_name
                 )
+                assert_readable_directory_in_configuration(typeshed_version_directory)
 
-            if not is_list_of_strings(self.other_critical_files):
-                raise InvalidConfiguration(
-                    "`critical_files` field must be a list of strings."
-                )
-        except InvalidConfiguration as error:
-            raise EnvironmentException(str(error))
+        # Validate elements of the search path.
+        for element in self._search_path:
+            assert_readable_directory_in_configuration(
+                element.path(), field_name="search_path"
+            )
+
+        if not is_list_of_strings(self.other_critical_files):
+            raise InvalidConfiguration(
+                "`critical_files` field must be a list of strings."
+            )
+
+        if not is_list_of_strings(self._do_not_ignore_errors_in):
+            raise InvalidConfiguration(
+                "`do_not_ignore_errors_in` field must be a list of strings."
+            )
 
     @property
     def version_hash(self) -> str:
@@ -485,6 +445,57 @@ class Configuration:
     def local_root(self) -> Optional[str]:
         return self._local_root
 
+    def get_existent_do_not_ignore_errors_in_paths(self) -> List[str]:
+        """
+        This is a separate method because we want to check for existing files
+        at the time this is called, not when the configuration is
+        constructed.
+        """
+        ignore_paths = [
+            _relativize_root(path, project_root=self.project_root, relative_root=None)
+            for path in self._do_not_ignore_errors_in
+        ]
+        paths = [path for path in ignore_paths if os.path.exists(path)]
+        non_existent_paths = set(ignore_paths) - set(paths)
+        if non_existent_paths:
+            LOG.debug(
+                "Filtering out nonexistent paths in `do_not_ignore_errors_in`: "
+                f"{non_existent_paths}"
+            )
+        return paths
+
+    def get_existent_ignore_all_errors_paths(self) -> List[str]:
+        """
+        This is a separate method because we want to check for existing files
+        at the time this is called, not when the configuration is
+        constructed.
+        """
+        expanded_ignore_paths = []
+        for path in self._ignore_all_errors:
+            rooted_path = _relativize_root(
+                path, project_root=self.project_root, relative_root=None
+            )
+            expanded = glob.glob(rooted_path)
+            if not expanded:
+                expanded_ignore_paths.append(path)
+            else:
+                expanded_ignore_paths += expanded
+
+        non_existent_ignore_paths = [
+            path for path in expanded_ignore_paths if not os.path.exists(path)
+        ]
+        if non_existent_ignore_paths:
+            LOG.warning(
+                "Nonexistent paths passed in to `ignore_all_errors`: "
+                f"`{non_existent_ignore_paths}`"
+            )
+        expanded_ignore_paths = [
+            path
+            for path in expanded_ignore_paths
+            if path not in non_existent_ignore_paths
+        ]
+        return expanded_ignore_paths
+
     def get_binary_version(self) -> Optional[str]:
         status = subprocess.run(
             [self.binary, "-version"], stdout=subprocess.PIPE, universal_newlines=True
@@ -519,10 +530,12 @@ class Configuration:
                 dot_pyre_directory=self.dot_pyre_directory,
             )
             paths_to_ignore = list(local_root.parents) + [local_root]
-            for ignore_element in parent_local_configuration.ignore_all_errors:
+            for (
+                ignore_element
+            ) in parent_local_configuration.get_existent_ignore_all_errors_paths():
                 if Path(ignore_element).resolve() in paths_to_ignore:
                     excluded_from_parent = True
-        except EnvironmentException as error:
+        except InvalidConfiguration as error:
             parent_error = error
 
         if not excluded_from_parent:
@@ -534,19 +547,19 @@ class Configuration:
                 "into a single configuration, or split the parent configuration to "
                 "avoid inconsistent errors."
             )
-            raise EnvironmentException(error_message)
+            raise InvalidConfiguration(error_message)
         elif parent_error:
-            raise EnvironmentException(parent_error)
+            raise InvalidConfiguration(str(parent_error))
 
     def _check_read_local_configuration(self, path: str) -> None:
         if not os.path.exists(path):
-            raise EnvironmentException(
+            raise InvalidConfiguration(
                 f"Local configuration path `{path}` does not exist."
             )
 
         local_configuration = os.path.join(path, LOCAL_CONFIGURATION_FILE)
         if not os.path.exists(local_configuration):
-            raise EnvironmentException(
+            raise InvalidConfiguration(
                 f"Local configuration directory `{path}` does not contain "
                 f"a `{LOCAL_CONFIGURATION_FILE}` file."
             )
@@ -554,7 +567,7 @@ class Configuration:
         self._check_nested_configurations(path)
         self._read(local_configuration)
         if not self.source_directories and not self.targets:
-            raise EnvironmentException(
+            raise InvalidConfiguration(
                 f"Local configuration `{local_configuration}` does not specify "
                 " any sources to type check."
             )
@@ -596,7 +609,7 @@ class Configuration:
                 ignore_all_errors = configuration.consume(
                     "ignore_all_errors", default=[]
                 )
-                self.ignore_all_errors.extend(ignore_all_errors)
+                self._ignore_all_errors.extend(ignore_all_errors)
 
                 ignore_infer = configuration.consume("ignore_infer", default=[])
                 self.ignore_infer.extend(ignore_infer)
@@ -689,6 +702,12 @@ class Configuration:
                     "critical_files", default=[]
                 )
 
+                do_not_ignore_errors_in = configuration.consume(
+                    "do_not_ignore_errors_in", default=[]
+                )
+                assert isinstance(do_not_ignore_errors_in, list)
+                self._do_not_ignore_errors_in.extend(do_not_ignore_errors_in)
+
                 # Warn on deprecated fields.
                 for deprecated_field in configuration._deprecated.keys():
                     configuration.consume(deprecated_field)
@@ -706,7 +725,7 @@ class Configuration:
             # We error elsewhere if there weren't enough parameters passed into pyre.
             self._expand_relative_paths(configuration_directory)
         except json.JSONDecodeError as error:
-            raise EnvironmentException(
+            raise InvalidConfiguration(
                 "Configuration file at `{}` is invalid: {}.".format(path, str(error))
             )
 
@@ -720,8 +739,8 @@ class Configuration:
         if logger:
             self.logger = expand_relative_path(root, logger)
 
-        self.ignore_all_errors = [
-            expand_relative_path(root, path) for path in self.ignore_all_errors
+        self._ignore_all_errors = [
+            expand_relative_path(root, path) for path in self._ignore_all_errors
         ]
 
         self.ignore_infer = [
@@ -748,6 +767,10 @@ class Configuration:
             expand_relative_path(root, path) for path in self.other_critical_files
         ]
 
+        self._do_not_ignore_errors_in = [
+            expand_relative_path(root, path) for path in self._do_not_ignore_errors_in
+        ]
+
     def _resolve_versioned_paths(self) -> None:
         version_hash = self.version_hash
         if not version_hash:
