@@ -102,21 +102,22 @@ def run_pyre(arguments: argparse.Namespace) -> ExitCode:
         configuration = configuration_module.Configuration.from_arguments(
             command_arguments.CommandArguments.from_arguments(arguments), Path(".")
         )
-        if configuration is None:
-            raise commands.ClientException(
-                "Cannot locate a `.pyre_configuration` file in the current directory "
-                "or any parent directories."
-            )
         if arguments.version:
             try:
-                binary_version = configuration.get_binary_version()
-                if binary_version:
-                    log.stdout.write(f"Binary version: {binary_version}\n")
+                if configuration is not None:
+                    binary_version = configuration.get_binary_version()
+                    if binary_version:
+                        log.stdout.write(f"Binary version: {binary_version}\n")
             except Exception:
                 pass
             log.stdout.write(f"Client version: {__version__}\n")
             exit_code = ExitCode.SUCCESS
         else:
+            if configuration is None:
+                raise commands.ClientException(
+                    "Cannot locate a `.pyre_configuration` file in the current directory "
+                    "or any parent directories."
+                )
             command = arguments.command(arguments, original_directory, configuration)
             log.start_logging_to_directory(
                 arguments.noninteractive, command.configuration.log_directory
@@ -142,7 +143,11 @@ def run_pyre(arguments: argparse.Namespace) -> ExitCode:
             raise FailedOutsideLocalConfigurationException(
                 exit_code, command, str(error)
             )
-    except (buck.BuckException, EnvironmentException) as error:
+    except (
+        buck.BuckException,
+        configuration_module.InvalidConfiguration,
+        EnvironmentException,
+    ) as error:
         if arguments.command == commands.Persistent.from_arguments:
             try:
                 commands.Persistent.run_null_server(timeout=3600 * 12)
@@ -155,11 +160,11 @@ def run_pyre(arguments: argparse.Namespace) -> ExitCode:
                 exit_code = ExitCode.SUCCESS
         else:
             client_exception_message = str(error)
-            exit_code = (
-                ExitCode.BUCK_ERROR
-                if isinstance(error, buck.BuckException)
-                else ExitCode.FAILURE
-            )
+            exit_code = ExitCode.FAILURE
+            if isinstance(error, buck.BuckException):
+                exit_code = ExitCode.BUCK_ERROR
+            elif isinstance(error, configuration_module.InvalidConfiguration):
+                exit_code = ExitCode.CONFIGURATION_ERROR
     except commands.ClientException as error:
         client_exception_message = str(error)
         exit_code = ExitCode.FAILURE
@@ -263,6 +268,9 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
     if not hasattr(arguments, "command"):
         _set_default_command(arguments)
 
+    # Special-case `pyre init` because it is not a `Command` like the others.
+    if arguments.command == commands.Initialize.from_arguments:
+        return arguments.command().run().exit_code()
     return _run_pyre_with_retry(arguments)
 
 
