@@ -14,6 +14,7 @@ from pathlib import Path
 from typing import Any, Dict, Iterator, List, Optional, Tuple, Union, cast
 
 import libcst
+import libcst.matchers as libcst_matchers
 
 from . import UserError, ast
 
@@ -43,16 +44,110 @@ class LineBreakTransformer(libcst.CSTTransformer):
             )
         return updated_node
 
+    @staticmethod
+    def basic_parenthesize(
+        node: libcst.CSTNode,
+        whitespace: Optional[libcst.BaseParenthesizableWhitespace] = None,
+    ) -> libcst.CSTNode:
+        if not hasattr(node, "lpar"):
+            return node
+        if whitespace:
+            return node.with_changes(
+                lpar=[libcst.LeftParen(whitespace_after=whitespace)],
+                rpar=[libcst.RightParen()],
+            )
+        return node.with_changes(lpar=[libcst.LeftParen()], rpar=[libcst.RightParen()])
+
+    def leave_Assert(
+        self, original_node: libcst.Assert, updated_node: libcst.Assert
+    ) -> libcst.Assert:
+        test = updated_node.test
+        if not test:
+            return updated_node
+        assert_whitespace = updated_node.whitespace_after_assert
+        if isinstance(assert_whitespace, libcst.ParenthesizedWhitespace):
+            return updated_node.with_changes(
+                test=LineBreakTransformer.basic_parenthesize(test, assert_whitespace),
+                whitespace_after_assert=libcst.SimpleWhitespace(value=" "),
+            )
+        return updated_node.with_changes(
+            test=LineBreakTransformer.basic_parenthesize(test)
+        )
+
     def leave_Assign(
         self, original_node: libcst.Assign, updated_node: libcst.Assign
     ) -> libcst.Assign:
         assign_value = updated_node.value
-        if hasattr(assign_value, "lpar"):
-            parenthesized_value = assign_value.with_changes(
-                lpar=[libcst.LeftParen()], rpar=[libcst.RightParen()]
+        assign_whitespace = updated_node.targets[-1].whitespace_after_equal
+        if libcst_matchers.matches(
+            assign_whitespace, libcst_matchers.ParenthesizedWhitespace()
+        ):
+            adjusted_target = updated_node.targets[-1].with_changes(
+                whitespace_after_equal=libcst.SimpleWhitespace(value=" ")
             )
-            return updated_node.with_changes(value=parenthesized_value)
-        return updated_node
+            updated_targets = list(updated_node.targets[:-1])
+            updated_targets.append(adjusted_target)
+            return updated_node.with_changes(
+                targets=tuple(updated_targets),
+                value=LineBreakTransformer.basic_parenthesize(
+                    assign_value, assign_whitespace
+                ),
+            )
+        return updated_node.with_changes(
+            value=LineBreakTransformer.basic_parenthesize(assign_value)
+        )
+
+    def leave_Del(
+        self, original_node: libcst.Del, updated_node: libcst.Del
+    ) -> libcst.Del:
+        delete_target = updated_node.target
+        delete_whitespace = updated_node.whitespace_after_del
+        if isinstance(delete_whitespace, libcst.ParenthesizedWhitespace):
+            return updated_node.with_changes(
+                target=LineBreakTransformer.basic_parenthesize(
+                    delete_target, delete_whitespace
+                ),
+                whitespace_after_del=libcst.SimpleWhitespace(value=" "),
+            )
+        return updated_node.with_changes(
+            target=LineBreakTransformer.basic_parenthesize(delete_target)
+        )
+
+    def leave_Raise(
+        self, original_node: libcst.Raise, updated_node: libcst.Raise
+    ) -> libcst.Raise:
+        exception = updated_node.exc
+        if not exception:
+            return updated_node
+        raise_whitespace = updated_node.whitespace_after_raise
+        if isinstance(raise_whitespace, libcst.ParenthesizedWhitespace):
+            return updated_node.with_changes(
+                exc=LineBreakTransformer.basic_parenthesize(
+                    exception, raise_whitespace
+                ),
+                whitespace_after_raise=libcst.SimpleWhitespace(value=" "),
+            )
+        return updated_node.with_changes(
+            exc=LineBreakTransformer.basic_parenthesize(exception)
+        )
+
+    def leave_Return(
+        self, original_node: libcst.Return, updated_node: libcst.Return
+    ) -> libcst.Return:
+        return_value = updated_node.value
+        if not return_value:
+            return updated_node
+        return_whitespace = updated_node.whitespace_after_return
+        if isinstance(return_whitespace, libcst.ParenthesizedWhitespace):
+            return updated_node.with_changes(
+                value=LineBreakTransformer.basic_parenthesize(
+                    return_value, return_whitespace
+                ),
+                whitespace_after_return=libcst.SimpleWhitespace(value=" "),
+            )
+        return updated_node.with_changes(
+            value=LineBreakTransformer.basic_parenthesize(return_value)
+        )
 
 
 class PartialErrorSuppression(Exception):
